@@ -1,5 +1,5 @@
 // build/run probes
-// Copyright (C) 2005-2019 Red Hat Inc.
+// Copyright (C) 2005-2021 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -116,8 +116,17 @@ make_any_make_cmd(systemtap_session& s, const string& dir, const string& target)
       // PR13847: suppress debuginfo creation by default
       "CONFIG_DEBUG_INFO=",
 
+      // linux 5.11 wants btf but no, baby, no
+      "CONFIG_DEBUG_INFO_BTF_MODULES=",
+      
       // RHBZ1321628: suppress stack validation; expected to be temporary
-      "CONFIG_STACK_VALIDATION=",
+      // "CONFIG_STACK_VALIDATION=",
+      
+      // PR28140 ... as of kernel 5.14-rc*, this is actively
+      // dangerous, because it skips the full objtool processing
+      // chain, and the resulting tracepoint call sites in the ko are
+      // not properly instrumented.  See also Linux commit
+      // ab3257042c2.
     };
 
   // PR10280: suppress symbol versioning to restrict to exact kernel version
@@ -230,7 +239,8 @@ compile_dyninst (systemtap_session& s)
       "-fvisibility=hidden", "-O2", "-I" + s.runtime_path, "-D__DYNINST__",
       "-Wall", WERROR, "-Wno-unused", "-Wno-strict-aliasing",
       "-Wno-pointer-to-int-cast", "-Wno-int-to-pointer-cast",
-      "-pthread", "-lrt", "-fPIC", "-shared",
+      "-Wno-pragmas", "-Wno-pointer-sign", "-Wno-format",
+      "-pthread", "-lrt", "-ldw", "-fPIC", "-shared",
     };
 
   // BZ855981/948279.  Since dyninst/runtime.h includes __sync_* calls,
@@ -241,8 +251,6 @@ compile_dyninst (systemtap_session& s)
   if (s.architecture == "i386")
     cmd.push_back("-march=i586");
 
-  cmd.push_back("-Wno-pragmas");
-
   for (size_t i = 0; i < s.c_macros.size(); ++i)
     cmd.push_back("-D" + s.c_macros[i]);
 
@@ -362,7 +370,7 @@ compile_pass (systemtap_session& s)
   output_exportconf(s, o2, "synchronize_sched", "STAPCONF_SYNCHRONIZE_SCHED");
   output_autoconf(s, o, cs, "autoconf-task-uid.c", "STAPCONF_TASK_UID", NULL);
   output_autoconf(s, o, cs, "autoconf-from_kuid_munged.c", "STAPCONF_FROM_KUID_MUNGED", NULL);
-  output_exportconf(s, o2, "get_mm_exe_file", "STAPCONF_GET_MM_EXE_FILE");
+  output_exportconf(s, o2, "get_mm_exe_file", "STAPCONF_GET_MM_EXE_FILE_EXPORTED");
   output_dual_exportconf(s, o2, "alloc_vm_area", "free_vm_area", "STAPCONF_VM_AREA");
   output_autoconf(s, o, cs, "autoconf-procfs-owner.c", "STAPCONF_PROCFS_OWNER", NULL);
   output_autoconf(s, o, cs, "autoconf-alloc-percpu-align.c", "STAPCONF_ALLOC_PERCPU_ALIGN", NULL);
@@ -376,11 +384,13 @@ compile_pass (systemtap_session& s)
   output_autoconf(s, o, cs, "autoconf-utrace-regset.c", "STAPCONF_UTRACE_REGSET", NULL);
   output_autoconf(s, o, cs, "autoconf-uprobe-get-pc.c", "STAPCONF_UPROBE_GET_PC", NULL);
   output_autoconf(s, o, cs, "autoconf-hlist-4args.c", "STAPCONF_HLIST_4ARGS", NULL);
+  output_autoconf(s, o, cs, "autoconf-get-kretprobe.c", "STAPCONF_GET_KRETPROBE", NULL);
   output_exportconf(s, o2, "tsc_khz", "STAPCONF_TSC_KHZ");
   output_exportconf(s, o2, "cpu_khz", "STAPCONF_CPU_KHZ");
   output_exportconf(s, o2, "__module_text_address", "STAPCONF_MODULE_TEXT_ADDRESS");
   output_exportconf(s, o2, "add_timer_on", "STAPCONF_ADD_TIMER_ON");
-
+  output_autoconf(s, o, cs, "autoconf-514-panic.c", "STAPCONF_514_PANIC", NULL);
+  
   output_dual_exportconf(s, o2, "probe_kernel_read", "probe_kernel_write", "STAPCONF_PROBE_KERNEL");
   output_autoconf(s, o, cs, "autoconf-hw_breakpoint_context.c",
 		  "STAPCONF_HW_BREAKPOINT_CONTEXT", NULL);
@@ -420,9 +430,9 @@ compile_pass (systemtap_session& s)
   output_exportconf(s, o2, "proc_create_data", "STAPCONF_PROC_CREATE_DATA");
   output_autoconf(s, o, cs, "autoconf-proc_ops.c", "STAPCONF_PROC_OPS", NULL);
   output_exportconf(s, o2, "PDE_DATA", "STAPCONF_PDE_DATA");
+  output_autoconf(s, o, cs, "autoconf-pde_data.c", "STAPCONF_PDE_DATA2", NULL);
   output_autoconf(s, o, cs, "autoconf-module-sect-attrs.c", "STAPCONF_MODULE_SECT_ATTRS", NULL);
-  output_exportconf(s, o2, "kernel_read_file_from_path", "STAPCONF_KERNEL_READ_FILE_FROM_PATH");
-  output_autoconf(s, o, cs, "autoconf-kernel_read_file_from_path-offset.c", "STAPCONF_KERNEL_READ_FILE_FROM_PATH_OFFSET", NULL);
+  output_autoconf(s, o, cs, "autoconf-kernel_read-new-args.c", "STAPCONF_KERNEL_READ_NEW_ARGS", NULL);
   output_autoconf(s, o, cs, "autoconf-utrace-via-tracepoints.c", "STAPCONF_UTRACE_VIA_TRACEPOINTS", NULL);
   output_autoconf(s, o, cs, "autoconf-task_work-struct.c", "STAPCONF_TASK_WORK_STRUCT", NULL);
   output_autoconf(s, o, cs, "autoconf-vm-area-pte.c", "STAPCONF_VM_AREA_PTE", NULL);
@@ -517,10 +527,19 @@ compile_pass (systemtap_session& s)
   output_autoconf(s, o, cs, "autoconf-atomic_fetch_add_unless.c",
 		  "STAPCONF_ATOMIC_FETCH_ADD_UNLESS", NULL);
   output_autoconf(s, o, cs, "autoconf-lockdown-debugfs.c", "STAPCONF_LOCKDOWN_DEBUGFS", NULL);
+  output_autoconf(s, o, cs, "autoconf-lockdown-kernel.c", "STAPCONF_LOCKDOWN_KERNEL", NULL);
+  output_autoconf(s, o, cs, "autoconf-hlist_add_tail_rcu.c",
+		  "STAPCONF_HLIST_ADD_TAIL_RCU", NULL);
+  output_autoconf(s, o, cs, "autoconf-files_lookup_fd_raw.c",
+                  "STAPCONF_FILES_LOOKUP_FD_RAW", NULL);
+  output_autoconf(s, o, cs, "autoconf-task-state.c", "STAPCONF_TASK_STATE", NULL);
   
   // used by runtime/linux/netfilter.c
   output_exportconf(s, o2, "nf_register_hook", "STAPCONF_NF_REGISTER_HOOK");
 
+  // runtime/linux/kprobes.c
+  output_exportconf(s, o2, "module_mutex", "STAPCONF_MODULE_MUTEX");
+    
   // used by tapset/linux/ioblock.stp
   output_exportconf(s, o2, "disk_get_part", "STAPCONF_DISK_GET_PART");
 
@@ -556,7 +575,11 @@ compile_pass (systemtap_session& s)
   o << "EXTRA_CFLAGS += -freorder-blocks" << endl; // improve on -Os
 
   // Generate eh_frame for self-backtracing
-  o << "EXTRA_CFLAGS += -fasynchronous-unwind-tables" << endl;
+  // FIXME Work around the issue with riscv kernel modules not being
+  // loadable with asynchronous unwind tables due to R_RISCV_32_PCREL
+  // relocations.
+  if (s.architecture != "riscv")
+    o << "EXTRA_CFLAGS += -fasynchronous-unwind-tables" << endl;
 
   // We used to allow the user to override default optimization when so
   // requested by adding a -O[0123s] so they could determine the
@@ -567,10 +590,6 @@ compile_pass (systemtap_session& s)
 
   // o << "CFLAGS += -fno-unit-at-a-time" << endl;
 
-  // 256^W512 bytes should be enough for anybody
-  // XXX this doesn't validate varargs, per gcc bug #41633
-  o << "EXTRA_CFLAGS += $(call cc-option,-Wframe-larger-than=512)" << endl;
-
   // gcc 5.0.0-0.13.fc23 ipa-icf seems to consume gigacpu on stap-generated code
   o << "EXTRA_CFLAGS += $(call cc-option,-fno-ipa-icf)" << endl;
 
@@ -583,6 +602,12 @@ compile_pass (systemtap_session& s)
   // Accept extra diagnostic-suppression pragmas etc.
   o << "EXTRA_CFLAGS += -Wno-pragmas" << endl;
 
+  // Suppress gcc12 diagnostic bug in kernel-devel for 5.16ish
+  o << "EXTRA_CFLAGS += -Wno-infinite-recursion" << endl;
+
+  // Suppress gcc12 diagnostic about STAP_KPROBE_PROBE_STR_* null checks
+  o << "EXTRA_CFLAGS += -Wno-address" << endl;
+  
   // PR25845: Recent gcc (seen on 9.3.1) warns fairly common 32-bit pointer-conversions:
   o << "EXTRA_CFLAGS += $(call cc-option,-Wno-pointer-to-int-cast)" << endl;
   o << "EXTRA_CFLAGS += $(call cc-option,-Wno-int-to-pointer-cast)" << endl;
@@ -956,6 +981,9 @@ make_bpf_run_command (systemtap_session& s, const string& remotedir,
   if (s.suppress_warnings)
     cmd.push_back("-w");
 
+  if (!s.cmd.empty())
+    cmd.insert(cmd.end(), { "-c", s.cmd });
+
   if (s.target_pid)
     cmd.insert(cmd.end(), { "-x", lex_cast(s.target_pid) });
 
@@ -1021,6 +1049,8 @@ make_tracequeries(systemtap_session& s, const map<string,string>& contents)
   omf << "CONFIG_MODULE_SIG := n" << endl;
   // PR23488: need to override this kconfig, else we get no useful struct decls
   omf << "CONFIG_DEBUG_INFO_REDUCED := " << endl;
+  // objtool is slow and uses a lot of memory, skip it since these modules aren't loaded
+  omf << "CONFIG_STACK_VALIDATION := " << endl;
   // PR18389: disable GCC's Identical Code Folding, since the stubs may look identical
   omf << "EXTRA_CFLAGS += $(call cc-option,-fno-ipa-icf)" << endl;
 
@@ -1100,6 +1130,9 @@ make_typequery_kmod(systemtap_session& s, const vector<string>& headers, string&
 
   // PR23488: need to override this kconfig, else we get no useful struct decls
   omf << "CONFIG_DEBUG_INFO_REDUCED := " << endl;
+
+  // objtool is slow and uses a lot of memory, skip it since these modules aren't loaded
+  omf << "CONFIG_STACK_VALIDATION := " << endl;
   
   // NB: We use -include instead of #include because that gives us more power.
   // Using #include searches relative to the source's path, which in this case
@@ -1116,15 +1149,20 @@ make_typequery_kmod(systemtap_session& s, const vector<string>& headers, string&
   omf << "obj-m := " + basename + ".o" << endl;
   omf.close();
 
-  // create our empty source file
+  // create our -nearly- empty source file
   string source(dir + "/" + basename + ".c");
   ofstream osrc(source.c_str());
+
+  // this is mandated by linux kbuild as of 5.11+
+  osrc << "#include <linux/module.h>" << endl;
+  osrc << "MODULE_LICENSE(\"GPL\");" << endl;
+  
   osrc.close();
 
   // make the module
   vector<string> make_cmd = make_make_cmd(s, dir);
   bool quiet = (s.verbose < 4);
-  int rc = run_make_cmd(s, make_cmd, quiet, quiet);
+  int rc = run_make_cmd(s, make_cmd, quiet, false /* stderr should be quiet PR27658 */);
   if (rc)
     s.set_try_server ();
   return rc;
@@ -1155,7 +1193,7 @@ make_typequery_umod(systemtap_session& s, const vector<string>& headers, string&
     cmd.insert(cmd.end(), { "-include", headers[i] });
 
   bool quiet = (s.verbose < 4);
-  int rc = stap_system (s.verbose, cmd, quiet, quiet);
+  int rc = stap_system (s.verbose, cmd, quiet, false /* stderr should be quiet PR27658 */);
   if (rc)
     s.set_try_server ();
   return rc;
@@ -1165,6 +1203,13 @@ make_typequery_umod(systemtap_session& s, const vector<string>& headers, string&
 int
 make_typequery(systemtap_session& s, string& module)
 {
+  // check our memoized cache first
+  if (s.typequery_memo.find(module) != s.typequery_memo.end())
+    {
+      module = s.typequery_memo.at(module);
+      return 0;
+    }
+    
   int rc;
   string new_module;
   vector<string> headers;
@@ -1192,6 +1237,10 @@ make_typequery(systemtap_session& s, string& module)
   else
       rc = make_typequery_umod(s, headers, new_module);
 
+  // memoize the result --- even if it failed (rc != 0), so as to avoid
+  // repeated attempts to rebuild the same thing
+  s.typequery_memo[module] = new_module;
+  
   if (!rc)
     module = new_module;
 
