@@ -58,6 +58,7 @@ struct uprobe_derived_probe_group;
 struct utrace_derived_probe_group;
 struct itrace_derived_probe_group;
 struct task_finder_derived_probe_group;
+struct vma_tracker_derived_probe_group;
 struct timer_derived_probe_group;
 struct netfilter_derived_probe_group;
 struct profile_derived_probe_group;
@@ -123,7 +124,7 @@ struct parse_error: public std::runtime_error
 };
 
 
-class symresolution_info;
+struct symresolution_info;
 
 struct systemtap_session
 {
@@ -239,7 +240,8 @@ public:
   bool monitor;
   int monitor_interval;
   int timeout; // in ms
-
+  std::map<std::string,std::string> typequery_memo;
+  
   enum
    { dump_none,               // no dumping requested
      dump_probe_types,        // dump standard tapset probes
@@ -291,6 +293,9 @@ public:
   bool modules_must_be_signed();
   void get_mok_info();
 
+  bool module_sign_given;
+  std::string module_sign_mok_path;
+
   // NB: It is very important for all of the above (and below) fields
   // to be cleared in the systemtap_session ctor (session.cxx).
 
@@ -393,6 +398,7 @@ public:
   utrace_derived_probe_group* utrace_derived_probes;
   itrace_derived_probe_group* itrace_derived_probes;
   task_finder_derived_probe_group* task_finder_derived_probes;
+  vma_tracker_derived_probe_group* vma_tracker_derived_probes;
   timer_derived_probe_group* timer_derived_probes;
   netfilter_derived_probe_group* netfilter_derived_probes;
   profile_derived_probe_group* profile_derived_probes;
